#include "Vector.h"

std::ostream &operator<<(std::ostream &out, const Vector &vec)
{
    if (vec.empty())
    {
        out << "[ ]";
        return out;
    }
    else
    {
        out << "[";
        for (int i = 0; i < vec.size - 1; i++)
            out << vec.data[i] << ", ";
        out << vec.data[vec.size - 1] << "]";
        return out;
    }
}

std::istream &operator>>(std::istream &in, Vector &vec)
{
    int elem;
    do
    {
        in >> elem;
        vec.push_back(elem);
    } while (std::cin.get() != '\n');

    return in;
}

Vector::Vector() : data(new int[2]), size(0), capacity(2) {}

Vector::Vector(const Vector &vec) : data(new int[vec.size]), size(vec.size), capacity(vec.size)
{
    for (int i = 0; i < size; i++)
        data[i] = vec.data[i];
}

Vector::Vector(int *arr, int num) : data(new int[num]), size(num), capacity(num)
{
    for (int i = 0; i < num; i++)
        data[i] = arr[i];
}

Vector::~Vector()
{
    delete[] data;
    data = nullptr;
    size = capacity = 0;
}

Vector &Vector::operator=(const Vector &vec)
{
    if (this != &vec)
    {
        size = vec.size;
        capacity = vec.capacity;
        data = new int[capacity];
        for (int i = 0; i < size; i++)
            data[i] = vec.data[i];
    }
    return *this;
}

int &Vector::operator[](int index) const
{
    if (index < 0 || index >= size)
    {
        std::cout << "Index out of range!" << std::endl;
        exit(0);
    }
    return data[index];
}

Vector Vector::operator+(const Vector &vec)
{
    if (this->size != vec.size)
    {
        std::cout << "Vectors to be added together must be of the same size!" << std::endl;
        exit(0);
    }
    for (int i = 0; i < size; i++)
        data[i] += vec.data[i];
    return *this;
}

Vector &Vector::operator++()
{
    for (int i = 0; i < size; i++)
        data[i]++;
    return *this;
}

const Vector Vector::operator++(int)
{
    Vector old = *this;
    for (int i = 0; i < size; i++)
        data[i]++;
    return old;
}

int Vector::at(int index) const
{
    if (index < 0 || index >= size)
    {
        std::cout << "Index out of range!" << std::endl;
        exit(0);
    }
    return data[index];
}

void Vector::push_back(int elem)
{
    if (size == capacity)
    {
        capacity *= 2;
        int *tmp = new int[capacity];
        for (int i = 0; i < size; i++)
            tmp[i] = data[i];
        delete[] data;
        data = tmp;
    }
    data[size++] = elem;
}

int Vector::pop_back()
{
    if (empty())
    {
        std::cout << "Vector's empty!" << std::endl;
        exit(0);
    }
    return data[--size];
}

void Vector::reserve(int new_capc)
{
    if (new_capc > capacity)
    {
        int *tmp = new int[new_capc];
        for (int i = 0; i < size; i++)
            tmp[i] = data[i];
        delete[] data;
        data = tmp;
        capacity = new_capc;
    }
}

void Vector::shrink_to_fit()
{
    if (size < capacity)
    {
        int *tmp = new int[size];
        for (int i = 0; i < size; i++)
            tmp[i] = data[i];
        delete[] data;
        data = tmp;
        capacity = size;
    }
}

void Vector::insert(int pos, int num, int elem)
{
    if (pos <= 0 || pos > size)
    {
        std::cout << "Invaild position to insert" << std::endl;
        exit(0);
    }
    if (num <= 0)
    {
        std::cout << "Number should be positive" << std::endl;
        exit(0);
    }
    if (size + num > capacity)
        reserve(size + num);
    for (int i = size - 1; i >= pos - 1; i--)
        data[i + num] = data[i];
    for (int i = pos - 1; i < pos + num - 1; i++)
        data[i] = elem;
    size += num;
}

void Vector::erase(int first, int last)
{
    if (empty())
    {
        std::cout << "Vector's empty" << std::endl;
        exit(0);
    }

    if (first > last || first < 1 || last > size)
    {
        std::cout << "Invalid Range" << std::endl;
        exit(0);
    }
    int new_size = size - (last - first + 1);
    first--;
    for (int i = last; i < size; ++i)
        data[first++] = data[i];
    size = new_size;
}

void Vector::erase(int index)
{
    if (empty())
    {
        std::cout << "Vector's empty" << std::endl;
        exit(0);
    }
    if (index < 1 || index > size)
    {
        std::cout << "Invalid Range" << std::endl;
        exit(0);
    }
    for (int i = index-1; i < size; i++)
        data[i] = data[i + 1];
    size--;
}

void Vector::swap(Vector &v1, Vector &v2)
{
    std::swap(v1.data, v2.data);
    std::swap(v1.size, v2.size);
    std::swap(v1.capacity, v2.capacity);
}
